# SPDX-FileCopyrightText: 2025-present zerubabeldinsa <zerubabel.dinsa@gmail.com>
#
# SPDX-License-Identifier: MIT
__version__ = "0.1.0"
__author__ = "zerubabeldinsa"
__email__ = "zerubabel.dinsa@gmail.com"
__license__ = "MIT"
__summary__ = "A Python package for analyzing investment portfolios, including using quantitative methods like the efficient frontier for calculations an optimal asset allocation strategy."